@@ -13,7 +13,7 @@ SEXP apply_probe_data (SEXP object, SEXP probes) {
 
   for (i = 0; i < nprobe; i++) {
     SET_ELEMENT(vals,i,eval(PROTECT(lang2(VECTOR_ELT(probes,i),data)),
-                            CLOENV(VECTOR_ELT(probes,i))));
+      CLOENV(VECTOR_ELT(probes,i))));
     if (!IS_NUMERIC(VECTOR_ELT(vals,i))) {
       errorcall(R_NilValue,"probe %ld returns a non-numeric result",i+1);
     }
@@ -26,54 +26,38 @@ SEXP apply_probe_data (SEXP object, SEXP probes) {
   return retval;
 }
 
-SEXP apply_probe_sim (SEXP object, SEXP nsim, SEXP params, SEXP seed,
-  SEXP probes, SEXP datval, SEXP rho) {
+SEXP apply_probe_sim (SEXP object, SEXP nsim, SEXP params,
+  SEXP probes, SEXP datval, SEXP gnsi) {
   int nprotect = 0;
-  SEXP y, obs, times, t0, call, names;
-  SEXP retval, val, valnames, x;
-  int nprobe, nsims, nvars, ntimes, nvals;
+  SEXP x, y, names, sims;
+  SEXP returntype, retval, val, valnames;
+  int nprobe, nsims, nobs, ntimes, nvals;
   int xdim[2];
   double *xp, *yp;
   int p, s, i, j, k, len0 = 0, len = 0;
 
   PROTECT(nsim = AS_INTEGER(nsim)); nprotect++;
   if ((LENGTH(nsim)!=1) || (INTEGER(nsim)[0]<=0))
-    errorcall(R_NilValue,"'nsim' must be a positive integer");
+    errorcall(R_NilValue,"'nsim' must be a positive integer."); // #nocov
+
+  PROTECT(gnsi = duplicate(gnsi)); nprotect++;
 
   // 'names' holds the names of the probe values
   // we get these from a previous call to 'apply_probe_data'
   nprobe = LENGTH(probes);
   nvals = LENGTH(datval);
   PROTECT(names = GET_NAMES(datval)); nprotect++;
-  PROTECT(t0 = GET_SLOT(object,install("t0"))); nprotect++;
-  PROTECT(times = GET_SLOT(object,install("times"))); nprotect++;
+  PROTECT(returntype = NEW_INTEGER(1)); nprotect++;
+  *(INTEGER(returntype)) = 0;
+  PROTECT(sims = do_simulate(object,params,nsim,returntype,gnsi)); nprotect++;
+  PROTECT(y = VECTOR_ELT(sims,1)); nprotect++;
+  *(INTEGER(gnsi)) = 0;
 
-  // call 'simulate' to get simulated data sets
-  PROTECT(obs = NEW_LOGICAL(1)); nprotect++;
-  LOGICAL(obs)[0] = 1;		// we set obs=TRUE
-  PROTECT(call = LCONS(t0,R_NilValue)); nprotect++;
-  SET_TAG(call,install("t0"));
-  PROTECT(call = LCONS(times,call)); nprotect++;
-  SET_TAG(call,install("times"));
-  PROTECT(call = LCONS(obs,call)); nprotect++;
-  SET_TAG(call,install(".obs"));
-  PROTECT(call = LCONS(params,call)); nprotect++;
-  SET_TAG(call,install("params"));
-  PROTECT(call = LCONS(seed,call)); nprotect++;
-  SET_TAG(call,install("seed"));
-  PROTECT(call = LCONS(nsim,call)); nprotect++;
-  SET_TAG(call,install("nsim"));
-  PROTECT(call = LCONS(object,call)); nprotect++;
-  SET_TAG(call,install("object"));
-  PROTECT(call = LCONS(install("simulate.internal"),call)); nprotect++;
-  PROTECT(y = eval(call,rho)); nprotect++;
-
-  nvars = INTEGER(GET_DIM(y))[0];
+  nobs = INTEGER(GET_DIM(y))[0];
   nsims = INTEGER(GET_DIM(y))[1];
-  ntimes = INTEGER(GET_DIM(y))[2]; // recall that 'simulate' returns a value for time zero
-
+  ntimes = INTEGER(GET_DIM(y))[2];
   // set up temporary storage
-  xdim[0] = nvars; xdim[1] = ntimes;
+  xdim[0] = nobs; xdim[1] = ntimes;
   PROTECT(x = makearray(2,xdim)); nprotect++;
   setrownames(x,GET_ROWNAMES(GET_DIMNAMES(y)),2);
 
@@ -92,26 +76,26 @@ SEXP apply_probe_sim (SEXP object, SEXP nsim, SEXP params, SEXP seed,
 
       // copy the data from y[,s,] to x[,]
       xp = REAL(x);
-      yp = REAL(y)+nvars*s;
-      for (j = 0; j < ntimes; j++, yp += nvars*nsims) {
-        for (i = 0; i < nvars; i++, xp++) *xp = yp[i];
+      yp = REAL(y)+nobs*s;
+      for (j = 0; j < ntimes; j++, yp += nobs*nsims) {
+        for (i = 0; i < nobs; i++, xp++) *xp = yp[i];
       }
 
       // evaluate the probe on the simulated data
       PROTECT(val = eval(PROTECT(lang2(VECTOR_ELT(probes,p),x)),
-                         CLOENV(VECTOR_ELT(probes,p))));
+        CLOENV(VECTOR_ELT(probes,p))));
       if (!IS_NUMERIC(val)) {
-        errorcall(R_NilValue,"probe %ld returns a non-numeric result",p+1);
+        errorcall(R_NilValue,"probe %ld returns a non-numeric result.",p+1);
       }
 
       len = LENGTH(val);
       if (s == 0)
         len0 = len;
       else if (len != len0) {
-        errorcall(R_NilValue,"variable-sized results returned by probe %ld",p+1);
+        errorcall(R_NilValue,"variable-sized results returned by probe %ld.",p+1);
       }
       if (k+len > nvals)
-        errorcall(R_NilValue,"probes return different number of values on different datasets");
+        errorcall(R_NilValue,"probes return different number of values on different datasets.");
 
       xp = REAL(retval); yp = REAL(val);
       for (i = 0; i < len; i++) xp[s+nsims*(i+k)] = yp[i];
@@ -121,7 +105,7 @@ SEXP apply_probe_sim (SEXP object, SEXP nsim, SEXP params, SEXP seed,
 
   }
   if (k != nvals)
-    errorcall(R_NilValue,"probes return different number of values on different datasets");
+    errorcall(R_NilValue,"probes return different number of values on different datasets.");
 
   UNPROTECT(nprotect);
   return retval;
