@@ -29,14 +29,15 @@ static R_INLINE SEXP add_args (SEXP args, SEXP names)
 static R_INLINE SEXP eval_call (SEXP fn, SEXP args, double *p, int n)
 {
 
-  SEXP var = args, ans;
+  SEXP var = args, ans, ob;
   int v;
 
   for (v = 0; v < n; v++, p++, var=CDR(var)) *(REAL(CAR(var))) = *p;
 
-  PROTECT(ans = eval(LCONS(fn,args),CLOENV(fn)));
+  PROTECT(ob = LCONS(fn,args));
+  PROTECT(ans = eval(ob,CLOENV(fn)));
 
-  UNPROTECT(1);
+  UNPROTECT(2);
   return ans;
 
 }
